Imports System.Linq
Imports System.Text
Imports System.Threading.Tasks
Imports VideoRecorder.Shared
Imports Windows.Media.Capture
Imports Windows.Media.MediaProperties
Imports Windows.Storage
Imports Windows.Storage.Streams
Imports System

Namespace VideoRecorder.W8
	Public Class W8VideoRecorder
		Inherits VideoRecorderCore

		Private _videoCaptureDevice As MediaCapture

		' File details for storing and playing the recording.   
		Private _videoFileStream As IRandomAccessStream
		Private sfVideoFile As StorageFile

		Private _isoVideoFileName As String = "CameraMovie.mp4"

		Private _isStarted As Boolean

        'Public Shadows Event CaptureFailed As System.EventHandler
        Public Shadows Event CaptureFailed As EventHandler(Of GeneralErrorEventArgs)

		Public Overrides async Function InitializeVideoRecorderAsync() As Task
			_videoCaptureDevice = New MediaCapture
			AddHandler _videoCaptureDevice.Failed, AddressOf OnCaptureFailed

			await _videoCaptureDevice.InitializeAsync
		End Function

		Public Overrides async Function InitializeFileSinkAsync() As Task
			Dim isoStore As StorageFolder = ApplicationData.Current.LocalFolder
			sfVideoFile = await isoStore.CreateFileAsync(_isoVideoFileName, CreationCollisionOption.ReplaceExisting)
		End Function

		Public Overrides async Function InitializePlayerFileStreamAsync() As Task
			Dim isoStore As StorageFolder = ApplicationData.Current.LocalFolder
			sfVideoFile = await isoStore.GetFileAsync(_isoVideoFileName)
			_videoFileStream = await sfVideoFile.OpenAsync(FileAccessMode.Read)
		End Function

		Public Overrides ReadOnly Property VideoSource As Object
			Get
				Return _videoCaptureDevice
			End Get
		End Property

		Public Overrides ReadOnly Property VideoCaptureDevice As Object
			Get
				Return _videoCaptureDevice
			End Get
		End Property

		Public ReadOnly Property VideoFile As StorageFile
			Get
				Return sfVideoFile
			End Get
		End Property

		Public Overrides ReadOnly Property VideoFileStream As Object
			Get
				Return _videoFileStream
			End Get
		End Property

		Public Overrides ReadOnly Property IsVideoSourceStarted As Boolean
			Get
				Return _isStarted
			End Get
		End Property

		Public Overrides ReadOnly Property IsVideoSourceStopped As Boolean
			Get
				Return Not _isStarted
			End Get
		End Property

		Public Overrides async Function StartVideoSourceAsync() As Task
			await _videoCaptureDevice.StartPreviewAsync
			_isStarted = True
		End Function

		Public Overrides async Function StopVideoSourceAsync() As Task
			await _videoCaptureDevice.StopPreviewAsync
			_isStarted = False
		End Function

        Private Sub OnCaptureFailed(ByVal sender As Object, ByVal e As MediaCaptureFailedEventArgs)
            Dim args As New GeneralErrorEventArgs(e.Message)
            RaiseEvent CaptureFailed(sender, args)
        End Sub

		Public Overrides async Function StartVideoRecordingAsync() As Task
			' Connect fileSink to captureSource.
			If Me.VideoCaptureDevice IsNot Nothing AndAlso Me.IsVideoSourceStarted Then
				await Me.StopVideoSourceAsync
				await Me.InitializeFileSinkAsync
			End If

			' Begin recording.
			If Me.VideoCaptureDevice IsNot Nothing AndAlso Me.IsVideoSourceStopped Then
				await Me.StartVideoSourceAsync

				Dim mediaEncodingProfile As MediaEncodingProfile = MediaEncodingProfile.CreateMp4(VideoEncodingQuality.Auto)
				await _videoCaptureDevice.StartRecordToStorageFileAsync(mediaEncodingProfile, sfVideoFile)
			End If
		End Function

		Public Overrides async Function StopVideoRecordingAsync() As Task
			await _videoCaptureDevice.StopRecordAsync

			await Me.StopVideoSourceAsync
			Me.DisposeFileSink()
		End Function

		Public Overrides Sub DisposeFileSink()
			sfVideoFile = Nothing
		End Sub

		Public Overrides Sub DisposeVideoPlayer()
			_videoFileStream.Dispose()
			_videoFileStream = Nothing
			sfVideoFile = Nothing
		End Sub

		Public Overrides Sub DisposeVideoRecorder()
			_videoCaptureDevice = Nothing
		End Sub
	End Class
End Namespace
