Imports System.IO
Imports System.IO.IsolatedStorage
Imports System.Linq
Imports System.Text
Imports System.Threading.Tasks
Imports VideoRecorder.Shared
Imports Windows.Phone.Media.Capture
Imports Windows.Storage
Imports Windows.Storage.Streams
Imports System

Namespace sdkVideoRecorderCS
	Public Class WP8VideoRecorder
		Inherits VideoRecorderCore

		' Source and device for capturing video.
		Private _videoCaptureDevice As AudioVideoCaptureDevice

		' File details for storing the recording.      
		Private _isoVideoFile As IsolatedStorageFileStream
		Private _videoFileStream As IRandomAccessStream
		Private sfVideoFile As StorageFile

		Private _isoVideoFileName As String = "CameraMovie.mp4"

		Private _isStarted As Boolean

        Public Shadows Event CaptureFailed As EventHandler

		Public Overrides async Function InitializeVideoRecorderAsync() As Task
			Dim location As CameraSensorLocation = CameraSensorLocation.Back
			Dim captureResolutions = AudioVideoCaptureDevice.GetAvailableCaptureResolutions(location)

			_videoCaptureDevice = await AudioVideoCaptureDevice.OpenAsync(location, captureResolutions(0))

			AddHandler _videoCaptureDevice.RecordingFailed, AddressOf OnCaptureFailed
		End Function

		Public Overrides async Function InitializeFileSinkAsync() As Task
			Dim isoStore As StorageFolder = ApplicationData.Current.LocalFolder
			sfVideoFile = await isoStore.CreateFileAsync(_isoVideoFileName, CreationCollisionOption.ReplaceExisting)
		End Function

		Public Overrides async Function InitializePlayerFileStreamAsync() As Task
            _isoVideoFile = New IsolatedStorageFileStream(_isoVideoFileName, FileMode.Open, FileAccess.Read, IsolatedStorageFile.GetUserStoreForApplication)
		End Function

		Public Overrides ReadOnly Property VideoSource As Object
			Get
				Return _videoCaptureDevice
			End Get
		End Property

		Public Overrides ReadOnly Property VideoCaptureDevice As Object
			Get
				Return _videoCaptureDevice
			End Get
		End Property

		Public Overrides ReadOnly Property VideoFileStream As Object
			Get
				Return _isoVideoFile
			End Get
		End Property

		Public Overrides ReadOnly Property IsVideoSourceStarted As Boolean
			Get
				Return _isStarted
			End Get
		End Property

		Public Overrides ReadOnly Property IsVideoSourceStopped As Boolean
			Get
				Return Not _isStarted
			End Get
		End Property

        Public Overrides Async Function StartVideoSourceAsync() As Task

            _isStarted = True
        End Function

		Public Overrides async Function StopVideoSourceAsync() As Task
            _isStarted = False
		End Function

        Private Sub OnCaptureFailed(ByVal sender As Object, ByVal e As CaptureFailedEventArgs)
            Dim args As New GeneralErrorEventArgs(e.ErrorCode.ToString)
            RaiseEvent CaptureFailed(sender, args)
        End Sub

		Public Overrides async Function StartVideoRecordingAsync() As Task
			' Connect fileSink to captureSource.
			If Me.VideoCaptureDevice IsNot Nothing AndAlso Me.IsVideoSourceStarted Then
				await Me.StopVideoSourceAsync
				await Me.InitializeFileSinkAsync
			End If

			' Begin recording.
			If Me.VideoCaptureDevice IsNot Nothing AndAlso Me.IsVideoSourceStopped Then
				await Me.StartVideoSourceAsync

				_videoFileStream = await sfVideoFile.OpenAsync(FileAccessMode.ReadWrite)
				await _videoCaptureDevice.StartRecordingToStreamAsync(_videoFileStream)
			End If
		End Function

		Public Overrides async Function StopVideoRecordingAsync() As Task
			await _videoCaptureDevice.StopRecordingAsync

			await Me.StopVideoSourceAsync
			Me.DisposeFileSink()
		End Function

		Public Overrides Sub DisposeFileSink()
			_videoFileStream.Dispose()
			sfVideoFile = Nothing
		End Sub

		Public Overrides Sub DisposeVideoPlayer()
			_isoVideoFile = Nothing
		End Sub

		Public Overrides Sub DisposeVideoRecorder()
			_videoCaptureDevice = Nothing
		End Sub
	End Class
End Namespace
