Imports System.IO
Imports System.IO.IsolatedStorage
Imports System.Text
Imports System.Threading.Tasks
Imports System.Windows
Imports System.Windows.Media
Imports VideoRecorder.Shared

Namespace sdkVideoRecorderCS
	Public Class WP7VideoRecorder
		Inherits VideoRecorderCore

		' Source and device for capturing video.
		Private _captureSource As CaptureSource
		Private _videoCaptureDevice As VideoCaptureDevice

		' File details for storing the recording.        
		Private _isoVideoFile As IsolatedStorageFileStream
		Private _fileSink As FileSink
		Private _isoVideoFileName As String = "CameraMovie.mp4"

        Public Shadows Event CaptureFailed As EventHandler
        Public Overrides Async Function InitializeVideoRecorderAsync() As Task
            If _captureSource Is Nothing Then
                ' Create the VideoRecorder objects.
                _captureSource = New CaptureSource
                _fileSink = New FileSink

                _videoCaptureDevice = CaptureDeviceConfiguration.GetDefaultVideoCaptureDevice

                ' Add eventhandlers for captureSource.
                AddHandler _captureSource.CaptureFailed, AddressOf OnCaptureFailed
            End If
        End Function

        Public Overrides Async Function InitializeFileSinkAsync() As Task
            ' Connect the input and output of fileSink.
            _fileSink.CaptureSource = _captureSource
            _fileSink.IsolatedStorageFileName = _isoVideoFileName
        End Function

        Public Overrides Async Function InitializePlayerFileStreamAsync() As Task
            _isoVideoFile = New IsolatedStorageFileStream(_isoVideoFileName, FileMode.Open, FileAccess.Read, IsolatedStorageFile.GetUserStoreForApplication)
        End Function

        Public Overrides ReadOnly Property VideoSource As Object
            Get
                Return _captureSource
            End Get
        End Property

        Public Overrides ReadOnly Property VideoCaptureDevice As Object
            Get
                Return _videoCaptureDevice
            End Get
        End Property

        Public Overrides ReadOnly Property VideoFileStream As Object
            Get
                Return _isoVideoFile
            End Get
        End Property

        Public Overrides Async Function StartVideoSourceAsync() As Task
            If _captureSource IsNot Nothing Then
                _captureSource.Start()
            End If
        End Function

        Public Overrides Async Function StopVideoSourceAsync() As Task
            If _captureSource IsNot Nothing Then
                _captureSource.Stop()
            End If
        End Function

        Public Overrides ReadOnly Property IsVideoSourceStarted As Boolean
            Get
                Return _captureSource.State = CaptureState.Started
            End Get
        End Property

        Public Overrides ReadOnly Property IsVideoSourceStopped As Boolean
            Get
                Return _captureSource.State = CaptureState.Stopped
            End Get
        End Property

        Private Sub OnCaptureFailed(ByVal sender As Object, ByVal e As ExceptionRoutedEventArgs) Handles Me.CaptureFailed
            Dim args As New GeneralErrorEventArgs(e.ErrorException.Message)
            RaiseEvent CaptureFailed(sender, args)
        End Sub

        Public Overrides Async Function StartVideoRecordingAsync() As Task
            ' Connect fileSink to captureSource.
            If Me.VideoCaptureDevice IsNot Nothing AndAlso Me.IsVideoSourceStarted Then
                Await Me.StopVideoSourceAsync
                Await Me.InitializeFileSinkAsync
            End If

            ' Begin recording.
            If Me.VideoCaptureDevice IsNot Nothing AndAlso Me.IsVideoSourceStopped Then
                Await Me.StartVideoSourceAsync
            End If
        End Function

        Public Overrides Async Function StopVideoRecordingAsync() As Task
            Await Me.StopVideoSourceAsync
            Me.DisposeFileSink()
        End Function

        Public Overrides Sub DisposeFileSink()
            ' Disconnect fileSink.
            _fileSink.CaptureSource = Nothing
            _fileSink.IsolatedStorageFileName = Nothing
        End Sub

        Public Overrides Sub DisposeVideoPlayer()
            _isoVideoFile = Nothing
        End Sub

        Public Overrides Sub DisposeVideoRecorder()
            _captureSource = Nothing
            _videoCaptureDevice = Nothing
            _fileSink = Nothing
        End Sub
    End Class
End Namespace
